/*---------------------------------------------------------------------------------

Simple example for using my AdLib emulation

Copyright (C) 2010
Patrick Aalto (Pate)

This software is provided 'as-is', without any express or implied
warranty.  In no event will the authors be held liable for any
damages arising from the use of this software.

Permission is granted to anyone to use this software for any
purpose, including commercial applications, and to alter it and
redistribute it freely, subject to the following restrictions:

1.	The origin of this software must not be misrepresented; you
must not claim that you wrote the original software. If you use
this software in a product, an acknowledgment in the product
documentation would be appreciated but is not required.
2.	Altered source versions must be plainly marked as such, and
must not be misrepresented as being the original software.
3.	This notice may not be removed or altered from any source
distribution.



How to use:

The original PC program uses I/O ports 388h and 389h to communicate
with the AdLib card. Port 388h is used to send a command byte,
followed by sending a data byte to port 389h.

My emulation uses the FIFO system to send both a command and data
byte simultaneously, so that the command byte is in the upper byte
and data byte in the lower byte of the 16-bit value sent.

So, to use my AdLib emulation, simply send the combined command and
data bytes to ARM7 like this:

	fifoSendValue32(FIFO_ADLIB, (commandbyte<<8)|databyte );

For info about the AdLib commands, see here:
http://www.shipbrook.com/jeff/sb.html
	
That is all there is to it!

Hope you find use for this code, I am looking forward to seeing
AdLib emulation in Doom, Wolfenstein 3D and other PC games ported
to Nintendo DS! :-)

---------------------------------------------------------------------------------*/
#include <nds.h>
#include <stdio.h>

#define FIFO_ADLIB FIFO_USER_01

u16 exampledata[] = { 
0x0120,
0xBD00,					//  AM Depth=1, Vib Depth=1
// Channel 1 = violin sound
0x0800,					//	FM Music Mode, Key Split = 0
0x20F1,					//	Ch0Slot1: AmpMod=1,Vibrato=1,EGType=1,KSR=1,multiple=1
0x4080,					//	Ch0Slot1: Scaling=1.5db/oct, output level decreased by 0 dB
0x6063,					//	Ch0Slot1: Attack=6,Decay=3
0x8013,					//	Ch0Slot1: Sustain=-3dB,Release=3
0xE001,					//	Ch0Slot1: Waveform=01 = half sine
0x2360,					//	Ch0Slot2: AmpMod=0,Vibrato=1,EGType=1,KSR=0,multiple=0
0x4300,					//	Ch0Slot2: Scaling=0, output level decreased by 6+3+.75 dB
0x638F,					//	Ch0Slot2: Attack=8,Decay=7
0x8305,					//	Ch0Slot2: Sustain=-0dB,Release=5
0xE302,					//	Ch0Slot2: Waveform=10 = Positive Sine
0xC008,					//	Ch0: Feedback=4, DecayAlg=0 = modulator 1 modulates modulator 2
// Channel 7 = Snare drum sound
0x30C2,	         		// Ch7Op1: 00000010 = AM=on,Vib=on,EGType=percussive,KSR=0,mul=2=one octave above
0x5003,	         		// Ch7Op1: 00000011 = Scaling=no change,level=-(1.5+0.75)dB
0x70F0,	         		// Ch7Op1: 11110110 = Attack=F,Decay=0
0x9005,	         		// Ch7Op1: 00000101 = Sustain=0dB,Release=5
0xF000,	         		// Ch7Op1: 00000000 = Waveform=Sine
0x33C0,	         		// Ch7Op2: 11000000 = AM=on,Vib=on,EGType=percussive,KSR=0,mul=0=one octave below
0x5306,	         		// Ch7Op2: 00001110 = Scaling=no change,level=-(6+3+1.5)dB
0x73FC,	         		// Ch7Op2: 11111100 = Attack=F,Decay=C
0x9337,	         		// Ch7Op2: 00110111 = Sustain=-(6+3)dB,Release=7
0xF300,	         		// Ch7Op2: 00000000 = Waveform=Sine
0xC60C,	         		// Ch7:    00001100 = Feedback=6, alg=0
0x5300,	         		// Ch7Op2: 00000000 = Scaling=no change,level=0dB
};

//---------------------------------------------------------------------------------
int main(void) {
//---------------------------------------------------------------------------------
	int i, newKeys, oldKeys, xorKeys, freqA;

	oldKeys = xorKeys = 0;
	freqA = 0x2AE;		// Middle-C
	
	consoleDemoInit();  //setup the sub screen for printing

	iprintf("\x1b[43m\n\n\tSimple AdLib emulation demo\n\t by Patrick \"Pate\" Aalto\n\n\x1b[39m");
	// Wait a while so we know the ARM7 has started up the AdLib emulation.
	for (i=0; i < 10; i++ )
		swiWaitForVBlank();
	// Send AdLib commands to program sounds for channels 1 and 7.
	for (i=0; i < sizeof(exampledata)/2; i++ )
	{
		fifoSendValue32(FIFO_ADLIB, exampledata[i] );
	}
	// Banner...
	iprintf("\nPress key A to play a melodic\nsound, use the DPad up/down to\nchange the sound frequency.\n");
	iprintf("\nPress key B to play a snare\ndrum sound.\n");
	iprintf("\nNOTE! No$GBA does not play the\nsound correctly, so use real\nhardware!\n");

	while(1) {
		newKeys = keysCurrent();				// Get the current key state
		if ( newKeys != oldKeys )
		{
			// Key state was changed. Determine the difference.
			xorKeys = newKeys ^ oldKeys;
			oldKeys = newKeys;
		}
		else
			xorKeys = 0;
		if ( xorKeys & KEY_A )
		{
			if ( newKeys & KEY_A )				// Key-On, channel 1
			{
				fifoSendValue32(FIFO_ADLIB, 0xA000|(freqA&0xFF) );
				fifoSendValue32(FIFO_ADLIB, 0xB030|((freqA>>8)&2) );
			}
			else								// Key-Off, channel 1
				fifoSendValue32(FIFO_ADLIB, 0xB012 );
		}
		if ( xorKeys & KEY_B )
		{
			if ( newKeys & KEY_B )				// Key-On, channel 7
			{
				fifoSendValue32(FIFO_ADLIB, 0xA62E );
				fifoSendValue32(FIFO_ADLIB, 0xB62A );
			}
			else								// Key-Off, channel 7
				fifoSendValue32(FIFO_ADLIB, 0xB60A );
		}
		if ( newKeys & KEY_UP )					// Increase frequency, channel 1
		{
			if ( freqA < 0x3F0 )
				freqA+=1;
			if ( newKeys & KEY_A )				// If the key is on, send the new frequency
			{
				fifoSendValue32(FIFO_ADLIB, 0xA000|(freqA&0xFF) );
				fifoSendValue32(FIFO_ADLIB, 0xB030|((freqA>>8)&2) );
			}
		}
		if ( newKeys & KEY_DOWN )				// Decrease frequency, channel 1
		{
			if ( freqA > 0x100 )
				freqA-=1;
			if ( newKeys & KEY_A )				// If the key is on, send the new frequency
			{
				fifoSendValue32(FIFO_ADLIB, 0xA000|(freqA&0xFF) );
				fifoSendValue32(FIFO_ADLIB, 0xB030|((freqA>>8)&2) );
			}
		}
		swiWaitForVBlank();
	}

	return 0;
}
